@@ -20,11 +20,6 @@ sub accessor_generator {
   $_[0]->{accessor_generator}
 }
 
-sub construction_string {
-  my ($self) = @_;
-  $self->{construction_string} or 'bless({}, $class);'
-}
-
 sub install_delayed {
   my ($self) = @_;
   my $package = $self->{package};
@@ -42,17 +37,10 @@ sub generate_method {
     $spec->{$no_init}{init_arg} = $no_init;
   }
   local $self->{captures} = {};
-  my $body = '    my $class = shift;'."\n"
-            .'    $class = ref($class) if ref($class);'."\n";
-  $body .= $self->_handle_subconstructor($into, $name);
-  my $into_buildargs = $into->can('BUILDARGS');
-  if ( $into_buildargs && $into_buildargs != \&Moo::Object::BUILDARGS ) {
-      $body .= $self->_generate_args_via_buildargs;
-  } else {
-      $body .= $self->_generate_args;
-  }
+  my $body = '    my $class = shift;'."\n";
+  $body .= $self->_generate_args;
   $body .= $self->_check_required($spec);
-  $body .= '    my $new = '.$self->construction_string.";\n";
+  $body .= '    my $new = bless({}, $class);'."\n";
   $body .= $self->_assign_new($spec);
   if ($into->can('BUILD')) {
     require Method::Generate::BuildAll;
@@ -61,59 +49,21 @@ sub generate_method {
     );
   }
   $body .= '    return $new;'."\n";
-  if ($into->can('DEMOLISH')) {
-    require Method::Generate::DemolishAll;
-    Method::Generate::DemolishAll->new->generate_method($into);
-  }
   quote_sub
     "${into}::${name}" => $body,
     $self->{captures}, $quote_opts||{}
   ;
 }
 
-sub _handle_subconstructor {
-  my ($self, $into, $name) = @_;
-  if (my $gen = $self->{subconstructor_handler}) {
-    '    if ($class ne '.perlstring($into).') {'."\n".
-    $gen.
-    '    }'."\n";
-  } else {
-    ''
-  }
-}
-
 sub _cap_call {
   my ($self, $code, $captures) = @_;
   @{$self->{captures}}{keys %$captures} = values %$captures if $captures;
   $code;
 }
 
-sub _generate_args_via_buildargs {
-  my ($self) = @_;
-  q{    my $args = $class->BUILDARGS(@_);}."\n";
-}
-
-# inlined from Moo::Object - update that first.
 sub _generate_args {
   my ($self) = @_;
-  return <<'_EOA';
-    my $args;
-    if ( scalar @_ == 1 ) {
-        unless ( defined $_[0] && ref $_[0] eq 'HASH' ) {
-            die "Single parameters to new() must be a HASH ref"
-                ." data => ". $_[0] ."\n";
-        }
-        $args = { %{ $_[0] } };
-    }
-    elsif ( @_ % 2 ) {
-        die "The new() method for $class expects a hash reference or a key/value list."
-                . " You passed an odd number of arguments\n";
-    }
-    else {
-        $args = {@_};
-    }
-_EOA
-
+  q{    my $args = ref($_[0]) eq 'HASH' ? $_[0] : { @_ };}."\n";
 }
 
 sub _assign_new {
@@ -122,13 +72,11 @@ sub _assign_new {
   my $ag = $self->accessor_generator;
   NAME: foreach my $name (sort keys %$spec) {
     my $attr_spec = $spec->{$name};
+    next NAME unless defined(my $i = $attr_spec->{init_arg});
     unless ($ag->is_simple_attribute($name, $attr_spec)) {
-      next NAME unless defined($attr_spec->{init_arg})
-                         or $ag->has_eager_default($name, $attr_spec);
-      $test{$name} = $attr_spec->{init_arg};
+      $test{$name} = $i;
       next NAME;
     }
-    next NAME unless defined(my $i = $attr_spec->{init_arg});
     push @init, $i;
     push @slots, $name;
   }
